/*
DCBouncer: bouncer / proxy for Direct Connect networks.
Copyright (C) 2008-2017 poy <poy@123gen.com>

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

#ifndef DCBOUNCER_ADCHANDLER_H
#define DCBOUNCER_ADCHANDLER_H

#include <memory>

#include "Proxy.h"

#include "dcpp/AdcCommand.h"

class ADCHandler : public Proxy
{
public:
	explicit ADCHandler(EventHandler&, const Profile&);

private:
	typedef std::shared_ptr<dcpp::AdcCommand> shared_command_type;

	struct Client_info
	{
		Client_info() { }

		std::string sup;
		std::auto_ptr<dcpp::AdcCommand> inf;
	} client_info;

	struct Hub_info
	{
		Hub_info() : sid(0) { }

		void clear();

		std::string sup;
		std::string ssid;

		uint32_t sid;

		typedef std::map<uint32_t, shared_command_type> infs_type;
		infs_type infs;

		typedef std::vector<std::string> cmds_type;
		cmds_type cmds;
	} hub_info;

	struct Log
	{
		std::string message;
		time_t timestamp;
		std::string from;
		std::string to;
		std::string pm;
	};
	typedef std::vector<Log> logs_type;
	logs_type logs;
	typedef std::map<std::string, shared_command_type> log_infs_type;
	log_infs_type log_infs;

	void checkHubSup();
	std::string saveLogInf(const uint32_t);

	// Proxy
	char separator() const;
	void clientCommand(const std::string&);
	void hubConnected();
	void hubCommand(const std::string&);
	void clearHubInfo();
	void sendCommand(const std::string&, const std::string&);
	void sendMessage(const std::string&);
	void offlineDescription(const std::string&);
};

#endif
