/*
DCBouncer: bouncer / proxy for Direct Connect networks.
Copyright (C) 2008-2017 poy <poy@123gen.com>

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

#include "Client.h"

#include <algorithm>

#include "Crypto.h"
#include "EventHandler.h"

Client::authorized_type Client::authorized;

Client::Client(sockets::ISocketHandler& handler) :
sockets::TcpSocket(handler)
{
	SetDeleteByHandler();
}

void Client::Init()
{
	if (GetParent()->IsSSL())
	{
		EnableSSL();
	}
}

void Client::InitSSLServer()
{
	InitializeContext(std::string(), Crypto::getCertFile(), std::string());
}

void Client::OnAccept()
{
	if (authorized.empty() || std::find(authorized.begin(), authorized.end(), GetRemoteAddress()) != authorized.end())
	{
		Handler().clientConnected(this);
	}
	else
	{
		SetCloseAndDelete();
	}
}

void Client::OnRawData(const char* buf, size_t len)
{
	Handler().clientLine(GetParent()->GetPort(), std::string(buf, len));
}

void Client::OnDelete()
{
	Handler().clientDisconnected(GetParent()->GetPort());
}

const EventHandler& Client::Handler() const
{
	return static_cast<const EventHandler&>(sockets::TcpSocket::Handler());
}

EventHandler& Client::Handler()
{
	return static_cast<EventHandler&>(sockets::TcpSocket::Handler());
}
