/*
DCBouncer: bouncer / proxy for Direct Connect networks.
Copyright (C) 2008-2017 poy <poy@123gen.com>

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

#ifndef DCBOUNCER_EVENTHANDLER_H
#define DCBOUNCER_EVENTHANDLER_H

#include "Sockets/EventHandler.h"

#include "Profile.h"

class Client;
class Hub;
class Proxy;

class EventHandler : public sockets::EventHandler
{
public:
	EventHandler();
	~EventHandler();

	void add(sockets::Socket*, const Profile&);

	bool OkToAccept(sockets::Socket*);

	void clientConnected(Client*);
	void clientLine(port_t, const std::string&);
	void clientDisconnected(port_t);

	void hubConnected(port_t);
	void hubLine(port_t, const std::string&);
	void hubDisconnected(port_t);

	void connectHub(port_t);

	std::string getDataFile(port_t, const std::string&) const;
	std::string load(port_t, const std::string&) const;
	void save(port_t, const std::string&, const std::string&, bool) const;

private:
	struct Initializer
	{
		explicit Initializer(EventHandler&, const Profile&);
		~Initializer();

		void clientConnected(Client*);
		void clientLine(const std::string&);
		void clientDisconnected();

		void hubConnected();
		void hubLine(const std::string&);
		void hubDisconnected();

		void connectHub();

	private:
		Proxy* proxy;

		EventHandler& handler;
		const Profile profile;

		Client* client;
		Hub* hub;

		bool client_ok() const;
		bool hub_ok() const;
	};
	typedef std::map<port_t, Initializer*> initializers_type;
	initializers_type initializers;

	Initializer* getInitializer(port_t) const;

	bool secure(const Profile&) const;
	void setSecure(const Profile&) const;
};

#endif
