/*
DCBouncer: bouncer / proxy for Direct Connect networks.
Copyright (C) 2008-2017 poy <poy@123gen.com>

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

#include "FileSystem.h"

#include <cstdlib>
#include <sys/stat.h>

std::string FileSystem::bin_dir;

void FileSystem::init()
{
	char* res = getenv("_");
	if (res)
	{
		std::string temp = res;
		size_t i = temp.rfind('/');
		if (i != std::string::npos)
		{
			bin_dir = temp.substr(0, i + 1);
		}
	}
}

std::string FileSystem::absolute(const std::string& relative)
{
	if (!bin_dir.empty())
	{
		return bin_dir + relative;
	}

	char* res = realpath(relative.c_str(), 0);
	if (res)
	{
		std::string ret = res;
		free(res);
		return ret;
	}

	return "/usr/local/bin/" + relative;
}

void FileSystem::ensureDir(const std::string& path)
{
	size_t i = 0;
	while ((i = path.find('/', i)) != std::string::npos)
	{
		mkdir(path.substr(0, ++i).c_str(), S_IRWXU);
	}
}

bool FileSystem::exists(const std::string& path)
{
	struct stat s;
	return !stat(path.c_str(), &s);
}
