/*
DCBouncer: bouncer / proxy for Direct Connect networks.
Copyright (C) 2008-2017 poy <poy@123gen.com>

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

#ifndef DCBOUNCER_PROXY_H
#define DCBOUNCER_PROXY_H

#include "Sockets/IEventOwner.h"

#include "EventHandler.h"

class Proxy : private sockets::IEventOwner
{
	friend class EventHandler;

protected:
	explicit Proxy(EventHandler&, const Profile&);
	virtual ~Proxy() { }

	bool client_ok() const;
	bool hub_ok() const;

	std::string load(const std::string&) const;
	void save(const std::string&, const std::string&, bool = false) const;

	void sendCommands();
	bool checkCommand(const std::string&);

	const Profile profile;

	Client* client;
	Hub* hub;

private:
	EventHandler& handler;

	std::string client_buf;
	std::string hub_buf;

	long hub_conn_event;
	long offline_event;

	enum Commands
	{
		Commands_first = 1, // Reserve 0 for error cases.
		Reconnect = Commands_first,
		Restart = Reconnect + 2,
		Quit,
		Commands_last
	};
	typedef std::vector<std::string> commands_type;
	commands_type commands;

	size_t offline_percent;
	unsigned offline_hours;

	void onClientConnected(Client*);
	void onClientLine(const std::string&);
	void onClientDisconnected();

	void onHubConnected(Hub*);
	void onHubLine(const std::string&);
	void onHubDisconnected();

	void OnEvent(int);

	typedef void (Proxy::*onCommand_type)(const std::string&);
	void parseCommands(std::string&, const std::string&, onCommand_type);

	void clearEvent(long& id);

	void offlineDescription();
	void addOfflineEvent();
	void clearOfflineEvent();

	virtual char separator() const = 0;

	virtual void clientConnected() { }
	virtual void clientCommand(const std::string&) { }

	virtual void hubConnected() { }
	virtual void hubCommand(const std::string&) { }
	virtual void clearHubInfo() = 0;

	virtual void sendCommand(const std::string&, const std::string&) = 0;
	virtual void sendMessage(const std::string&) = 0;
	virtual void offlineDescription(const std::string&) = 0;
};

#endif
